﻿using System.ComponentModel;
using System.IO;

namespace ContactsApp.UI
{
    public enum CroppingType
    {
        Rectangle,
        Circle
    }

    public enum CroppingMode
    {
        Automatic,
        Manual
    }

    public class CroppedPicture : Control
    {
        [Category("Cropping")]
        [Browsable(true)]
        [DisplayName("Cropping Type")]
        public CroppingType ImageCroppingType { get; set; }

        [Category("Cropping")]
        [Browsable(true)]
        [DisplayName("Cropping Change")]
        [DefaultValue(5)]
        public int PercentChange { get; set;  }

        private Image? _image;
        private Brush? _imageBrush;
        [Category("Cropping")]
        [Browsable(true)]
        public Image? Picture
        {
            get => _image;
            set
            {
                if (value == _image) return;
                _imageBrush?.Dispose();
                _imageBrush = null;
                _image = value;
            }
        }

        private int _updatePeriod;
        [Category("Marquee")]
        [Browsable(true)]
        [DisplayName("Update Period")]
        [DefaultValue(100)]
        public virtual int UpdatePeriod
        {
            get => _updatePeriod;
            set
            {
                if (value <= 0)
                    throw new ArgumentOutOfRangeException("UpdatePeriod", "must be > 0");
                _updatePeriod = value;
            }
        }

        [Category("Cropping")]
        [Browsable(true)]
        [DisplayName("Cropping Mode")]
        public CroppingMode ImageCroppingMode { get; set; }

        private System.Windows.Forms.Timer timer;

        public CroppedPicture()
        {
            PercentChange = 5;
            UpdatePeriod = 100;
            ImageCroppingMode = CroppingMode.Manual;
            timer = new();
            timer.Tick += UpdateScale;
            _imageBrush = null;

            SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
        }

        private float scale = 1.0f;
        private float direction = 1.0f;
        private void UpdateScale(object? sender, EventArgs e)
        {
            scale -= direction * PercentChange / 100f;
            if (scale < 0.5f || scale > 1.1f)
                direction *= -1.0f;
            Refresh();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            if (!timer.Enabled && ImageCroppingMode == CroppingMode.Automatic)
                Animate();
            e.Graphics.Clear(BackColor);

            base.OnPaint(e);
            if (_imageBrush == null && _image != null)
                _imageBrush = new TextureBrush(new Bitmap(_image, Width, Height));

            if (_imageBrush != null)
            {
                var rect = GetScaledRectangle(e.Graphics.ClipBounds);
                switch (ImageCroppingType)
                {
                    case CroppingType.Rectangle:
                        e.Graphics.FillRectangle(_imageBrush, rect);
                        break;
                    case CroppingType.Circle:
                        e.Graphics.FillEllipse(_imageBrush, rect);
                        break;
                    default:
                        throw new ArgumentOutOfRangeException();
                }
            }
        }

        private void Animate()
        {
            timer.Interval = UpdatePeriod;
            timer.Start();
        }

        private RectangleF GetScaledRectangle(RectangleF rect)
        {
            var newWidht = rect.Width * scale;
            var newHeight = rect.Height * scale;
            var x = (rect.Width - newWidht) / 2;
            var y = (rect.Height - newHeight) / 2;

            return new RectangleF(x, y, newWidht, newHeight);
        }
    }
}
